/* 
 * Imu.c
 * 
 * Computing the rawdata from the MPU6050 IMU-Sensor.
 *
 * Created by Matthias Kesenheimer on 17/07/2012.
 * Copyright 2012 Matthias Kesenheimer. All rights reserved
 *
 * Functions:
 * 	ISR(TWIE_TWIM_vect)
 * 	void GYRO_calibrate()
 *	void ACC_calibrate()
 *	void Gyro_getADC()
 *	void Temp_getADC()
 *	void ACC_getADC()
 *	void annexCode()
 *	void rotateV(struct fp_vector *v, float* delta)
 *	int16_t _atan2(float y, float x)
 *	void getEstimatedAttitude()
 *	void computeIMU()
 */

//MasterTwi Interrupthandler
ISR(TWIE_TWIM_vect)
{
	TWI_MasterInterruptHandler(&twiMaster);
}

void GYRO_calibrate()
{
	static int16_t previousGyroADC[3] = {0,0,0};
	static int32_t g[3];
	uint8_t axis;
	
	//Es werden maxStepsGA-Werte aufsummiert und daraus der
	//Mittelwert bestimmt und danach in gyroZero geschrieben
	if (calibratingG>0) {
		for (axis = 0; axis < 3; axis++) {
			// Reset g[axis] at start of calibration
			if (calibratingG == maxStepsGA) g[axis]=0;
			// Sum up maxStepsGA readings
			g[axis] +=gyroADC[axis];
			// Clear global variables for next reading
			gyroADC[axis]=0;
			gyroZero[axis]=0;
			if (calibratingG == 1) {
				gyroZero[axis]=g[axis]/maxStepsGA;
				//blinkLED(10,15,1);
    		_delay_loop_2(2000);
    		//LED blinken lassen um anzuzeigen, dass der Quadro sich kalibriert
    		//TODO
    		//PORTA.OUTTGL |= (1<<PIN7);	// LED hinten (rot)
			}
		}
		
		//LED blinken lassen um anzuzeigen, dass der Quadro sich kalibriert
		//TODO
		//PORTA.OUTTGL |= (1<<PIN7);	// LED hinten (rot)
			
	calibratingG--;
	}
		//PORTA.OUTCLR |= (1<<PIN7);	// LED hinten (rot)		// wieder aus machen
	
	//jedesmal wenn GYRO_calibrate() aufgerufen wird,
	//werden die Gyro Werte um Gyrozero korrigiert
	for (axis = 0; axis < 3; axis++) {
		gyroADC[axis]  -= gyroZero[axis];
		//anti gyro glitch, limit the variation between two consecutive readings
		gyroADC[axis] = constrain(gyroADC[axis],previousGyroADC[axis]-800,previousGyroADC[axis]+800);    
		previousGyroADC[axis] = gyroADC[axis];
	}
}

void ACC_calibrate()
{
	
	if (calibratingA>0) {
		uint8_t axis;
		for (axis = 0; axis < 3; axis++) {
			// Reset calibrationSum[axis] at start of calibration
			if (calibratingA == maxStepsGA) calibrationSum[axis]=0;
			// Sum up maxStepsGA readings
			calibrationSum[axis] +=accADC[axis];
			
			// Clear global variables for next reading
			if(calibratingA != 1)
				accADC[axis]=0; //beim letzten Durchlauf, darf der letzte 
						//Eintrag nicht wieder geloescht werden
			conf.accZero[axis]=0;
		}
		
		// Calculate average, shift Z down by acc_1G and store values in EEPROM at end of calibration
		if (calibratingA == 1) {
			conf.accZero[ROLL]  = calibrationSum[ROLL]/maxStepsGA;
			conf.accZero[PITCH] = calibrationSum[PITCH]/maxStepsGA;
			conf.accZero[YAW]   = calibrationSum[YAW]/maxStepsGA-acc_1G;
			conf.angleTrim[ROLL]   = 0;
			conf.angleTrim[PITCH]  = 0;
			writeParams(1); // write accZero in EEPROM
		}
		
		//LED blinken lassen um anzuzeigen, dass der Quadro sich kalibriert
		//TODO
		//PORTA.OUTTGL |= (1<<PIN7);	// LED hinten (rot)
		calibratingA--;
	}
	//PORTA.OUTCLR |= (1<<PIN7);	// LED hinten (rot)		// wieder aus machen
		
	  #if defined(INFLIGHT_ACC_CALIBRATION)
	  static int32_t b[3];
	  static int16_t accZero_saved[3]  = {0,0,0};
	  static int16_t  angleTrim_saved[2] = {0, 0};
	  //Saving old zeropoints before measurement
	  if (InflightcalibratingA==50) {
		  accZero_saved[ROLL]  = conf.accZero[ROLL] ;
		  accZero_saved[PITCH] = conf.accZero[PITCH];
		  accZero_saved[YAW]   = conf.accZero[YAW] ;
		  angleTrim_saved[ROLL] = conf.angleTrim[ROLL] ;
		  angleTrim_saved[PITCH] = conf.angleTrim[PITCH] ;
	  }
	  if (InflightcalibratingA>0) {
		  for (uint8_t axis = 0; axis < 3; axis++) {
			  // Reset a[axis] at start of calibration
			  if (InflightcalibratingA == 50) b[axis]=0;
			  // Sum up 50 readings
			  b[axis] +=accADC[axis];
			  // Clear global variables for next reading
			  accADC[axis]=0;
			  conf.accZero[axis]=0;
		  }
		  //all values are measured
		  if (InflightcalibratingA == 1) {
			  AccInflightCalibrationActive = 0;
			  AccInflightCalibrationMeasurementDone = 1;
			  #if defined(BUZZER)
			  toggleBeep = 2;      //buzzer for indicatiing the end of calibration
			  #endif
			  // recover saved values to maintain current flight behavior until new values are transferred
			  conf.accZero[ROLL]  = accZero_saved[ROLL] ;
			  conf.accZero[PITCH] = accZero_saved[PITCH];
			  conf.accZero[YAW]   = accZero_saved[YAW] ;
			  conf.angleTrim[ROLL] = angleTrim_saved[ROLL] ;
			  conf.angleTrim[PITCH] = angleTrim_saved[PITCH] ;
		  }
		  InflightcalibratingA--;
	  }
	  // Calculate average, shift Z down by acc_1G and store values in EEPROM at end of calibration
	  if (AccInflightCalibrationSavetoEEProm == 1){  //the copter is landed, disarmed and the combo has been done again
		  AccInflightCalibrationSavetoEEProm = 0;
		  conf.accZero[ROLL]  = b[ROLL]/50;
		  conf.accZero[PITCH] = b[PITCH]/50;
		  conf.accZero[YAW]   = b[YAW]/50-acc_1G; // for nunchuk 200=1G
		  conf.angleTrim[ROLL]   = 0;
		  conf.angleTrim[PITCH]  = 0;
		  writeParams(1); // write accZero in EEPROM
	  }
	  #endif	

	accADC[ROLL]  -=  conf.accZero[ROLL] ; //Die ACC Werte werden um acczero korrigiert
	accADC[PITCH] -=  conf.accZero[PITCH];
	accADC[YAW]   -=  conf.accZero[YAW] ;
	
}

void Gyro_getADC()
{
	ra = 0x43; //Startregister der Winkelbeschleunigungswerte
	TWI_MasterWriteRead(&twiMaster, SLAVE_ADDRESS, &ra, 1, 6);
	while (twiMaster.status != TWIM_STATUS_READY){}
	gyroADC[PITCH]	= - ((twiMaster.readData[0] << 8) + twiMaster.readData[1])/4;
	gyroADC[ROLL]	= + ((twiMaster.readData[2] << 8) + twiMaster.readData[3])/4;
	gyroADC[YAW]	= - ((twiMaster.readData[4] << 8) + twiMaster.readData[5])/4;

	GYRO_calibrate();
}

void Temp_getADC()
{
	ra = 0x41; //Startregister der Temperaturmessung
	TWI_MasterWriteRead(&twiMaster, SLAVE_ADDRESS, &ra, 1, 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
	temperatur = ((twiMaster.readData[0] << 8) + twiMaster.readData[1]);
}

void ACC_getADC()
{	
	//static int16_t accx_max = -32000;
	//static int16_t accx_min =  32000;
	//PORTA.OUTSET |= (1<<PIN7);	// LED hinten (rot)
	
	ra = 0x3b; //Startregister der Beschleunigungswerte
	TWI_MasterWriteRead(&twiMaster, SLAVE_ADDRESS, &ra, 1, 6); //Senden der Registeraddresse und auf Antwort warten, &ra = Adresse der Variable ra bergeben
	while (twiMaster.status != TWIM_STATUS_READY){} //Warten bis die Kommunikation abgeschlossen ist
	
	accADC[ROLL]	= - ((twiMaster.readData[0] << 8) + twiMaster.readData[1])/8; //empfangene 8 Bit Werte in einer 16 Bit Variable speichern
	accADC[PITCH]	= - ((twiMaster.readData[2] << 8) + twiMaster.readData[3])/8;
	accADC[YAW]		= + ((twiMaster.readData[4] << 8) + twiMaster.readData[5])/8;
	
	//if (accx_max < accADC[ROLL]) accx_max = accADC[ROLL];	// Maximum ermitteln
	//if (accx_min > accADC[ROLL]) accx_min = accADC[ROLL];	// Minimum ermitteln
			//
			//debug[0] = accx_min;
			//debug[1] = accx_max;

	ACC_calibrate(); //Korrektur der Werte um den anfnglich berechneten Offsetwerten
}

void annexCode() { // this code is excetuted at each loop and won't interfere with control loop if it lasts less than 650 microseconds
	static uint32_t calibratedAccTime;
	uint16_t tmp,tmp2;
	uint8_t axis,prop1,prop2;

	#define BREAKPOINT 1500
	// PITCH & ROLL only dynamic PID adjustemnt,  depending on throttle value
	if   (rcData[THROTTLE]<BREAKPOINT) {
		prop2 = 100;
		} else {
		if (rcData[THROTTLE]<2000) {
			prop2 = 100 - (uint16_t)conf.dynThrPID*(rcData[THROTTLE]-BREAKPOINT)/(2000-BREAKPOINT);
			} else {
			prop2 = 100 - conf.dynThrPID;
		}
	}

	for(axis=0;axis<3;axis++) {
		tmp = min(abs(rcData[axis]-MIDRC),500);
		#if defined(DEADBAND)
		if (tmp>DEADBAND) { tmp -= DEADBAND; }
		else { tmp=0; }
		#endif
		if(axis!=2) { //ROLL & PITCH
			tmp2 = tmp/100;
			rcCommand[axis] = lookupPitchRollRC[tmp2] + (tmp-tmp2*100) * (lookupPitchRollRC[tmp2+1]-lookupPitchRollRC[tmp2]) / 100;
			prop1 = 100-(uint16_t)conf.rollPitchRate*tmp/500;
			prop1 = (uint16_t)prop1*prop2/100;
			} else {      // YAW
			rcCommand[axis] = tmp;
			prop1 = 100-(uint16_t)conf.yawRate*tmp/500;
		}
		dynP8[axis] = (uint16_t)conf.P8[axis]*prop1/100;
		dynD8[axis] = (uint16_t)conf.D8[axis]*prop1/100;
		if (rcData[axis]<MIDRC) rcCommand[axis] = -rcCommand[axis];

	}
	tmp = constrain(rcData[THROTTLE],MINCHECK,2000);
	tmp = (uint32_t)(tmp-MINCHECK)*1000/(2000-MINCHECK); // [MINCHECK;2000] -> [0;1000]
	tmp2 = tmp/100;
	rcCommand[THROTTLE] = lookupThrottleRC[tmp2] + (tmp-tmp2*100) * (lookupThrottleRC[tmp2+1]-lookupThrottleRC[tmp2]) / 100; // [0;1000] -> expo -> [MINTHROTTLE;MAXTHROTTLE]

	if(f.HEADFREE_MODE) { //to optimize
		float radDiff = (heading - headFreeModeHold) * 0.0174533f; // where PI/180 ~= 0.0174533
		float cosDiff = cos(radDiff);
		float sinDiff = sin(radDiff);
		int16_t rcCommand_PITCH = rcCommand[PITCH]*cosDiff + rcCommand[ROLL]*sinDiff;
		rcCommand[ROLL] =  rcCommand[ROLL]*cosDiff - rcCommand[PITCH]*sinDiff;
		rcCommand[PITCH] = rcCommand_PITCH;
	}

	#if defined(POWERMETER_HARD)
	uint16_t pMeterRaw;               // used for current reading
	static uint16_t psensorTimer = 0;
	if (! (++psensorTimer % PSENSORFREQ)) {
		pMeterRaw =  analogRead(PSENSORPIN);
		powerValue = ( PSENSORNULL > pMeterRaw ? PSENSORNULL - pMeterRaw : pMeterRaw - PSENSORNULL); // do not use abs(), it would induce implicit cast to uint and overrun
		if ( powerValue < 333) {  // only accept reasonable values. 333 is empirical
			#ifdef LCD_TELEMETRY
			if (powerValue > powerMax) powerMax = powerValue;
			#endif
			} else {
			powerValue = 333;
		}
		pMeter[PMOTOR_SUM] += (uint32_t) powerValue;
	}
	#endif

	#if defined(VBAT)
		static uint8_t vbatTimer = 0; 
		static uint8_t ind = 0;
		uint16_t vbatRaw = 0;
		static uint16_t vbatRawArray[8];
		
		if (! (++vbatTimer % VBATFREQ)) {	
			vbatRawArray[(ind++)%8] = analogRead(V_BATPIN);
			
			for (uint8_t i=0;i<8;i++) vbatRaw += (vbatRawArray[i]);
			//debug[3] = vbatRaw;
			
			vbat = vbatRaw / (VBATSCALE/2);                  // result is Vbatt in 0.1V steps
			//debug[2] = vbat;
			}
		if ( ( (vbat>VBATLEVEL1_3S)
			#if defined(POWERMETER)
			&& ( (pMeter[PMOTOR_SUM] < pAlarm) || (pAlarm == 0) )
			#endif
		)  || (NO_VBAT>vbat)                              ) // ToLuSe
		{                                          // VBAT ok AND powermeter ok, buzzer off
			buzzerFreq = 0;
			#if defined(POWERMETER)
		} else if (pMeter[PMOTOR_SUM] > pAlarm) {                             // sound alarm for powermeter
			buzzerFreq = 4;
			#endif
		} else if (vbat>VBATLEVEL2_3S) buzzerFreq = 1;
		else if (vbat>VBATLEVEL3_3S)   buzzerFreq = 2;
		else                           buzzerFreq = 4;
	#endif
	
	#if defined(BUZZER)
	buzzer(buzzerFreq); // external buzzer routine that handles buzzer events globally now
	#endif

	if ( (calibratingA>0 && ACC ) || (calibratingG>0) ) { // Calibration phasis
		LEDPIN_TOGGLE;
		} else {
		if (f.ACC_CALIBRATED) {LEDPIN_OFF;}
		if (f.ARMED) {LEDPIN_ON;}
	}

	#if defined(LED_RING)
	static uint32_t LEDTime;
	if ( currentTime > LEDTime ) {
		LEDTime = currentTime + 50000;
		i2CLedRingState();
	}
	#endif

	#if defined(LED_FLASHER)
	auto_switch_led_flasher();
	#endif

	if ( currentTime > calibratedAccTime ) {
		if (! f.SMALL_ANGLES_25) {
			// the multi uses ACC and is not calibrated or is too much inclinated
			f.ACC_CALIBRATED = 0;
			LEDPIN_TOGGLE;
			calibratedAccTime = currentTime + 500000;
			} else {
			f.ACC_CALIBRATED = 1;
		}
	}

	#if defined(GPS_PROMINI)
		if(GPS_Enable == 0) {serialCom();}
	#else
		serialCom();
	#endif

	#if defined(POWERMETER)
	intPowerMeterSum = (pMeter[PMOTOR_SUM]/PLEVELDIV);
	intPowerTrigger1 = conf.powerTrigger1 * PLEVELSCALE;
	#endif

	#ifdef LCD_TELEMETRY_AUTO
	static char telemetryAutoSequence []  = LCD_TELEMETRY_AUTO;
	static uint8_t telemetryAutoIndex = 0;
	static uint16_t telemetryAutoTimer = 0;
	if ( (telemetry_auto) && (! (++telemetryAutoTimer % LCD_TELEMETRY_AUTO_FREQ) )  ){
		telemetry = telemetryAutoSequence[++telemetryAutoIndex % strlen(telemetryAutoSequence)];
		LCDclear(); // make sure to clear away remnants
	}
	#endif
	#ifdef LCD_TELEMETRY
	static uint16_t telemetryTimer = 0;
	if (! (++telemetryTimer % LCD_TELEMETRY_FREQ)) {
		#if (LCD_TELEMETRY_DEBUG+0 > 0)
		telemetry = LCD_TELEMETRY_DEBUG;
		#endif
		if (telemetry) lcd_telemetry();
	}
	#endif

	#if GPS & defined(GPS_LED_INDICATOR)
	static uint32_t GPSLEDTime;
	if ( currentTime > GPSLEDTime && (GPS_numSat >= 5)) {
		GPSLEDTime = currentTime + 150000;
		LEDPIN_TOGGLE;
	}
	#endif

	#if defined(LOG_VALUES) && (LOG_VALUES == 2)
	if (cycleTime > cycleTimeMax) cycleTimeMax = cycleTime; // remember highscore
	if (cycleTime < cycleTimeMin) cycleTimeMin = cycleTime; // remember lowscore
	#endif
	#ifdef LCD_TELEMETRY
	if (f.ARMED) armedTime += (uint32_t)cycleTime;
	#if BARO
	if (!f.ARMED) {
		BAROaltStart = BaroAlt;
		BAROaltMax = BaroAlt;
		} else {
		if (BaroAlt > BAROaltMax) BAROaltMax = BaroAlt;
	}
	#endif
	#endif
}

//der Ortsvektor wird um die Eulerwinkel (Yaw-Pitch-Roll = Psi, Phi, Theta) gemaess kleiner Winkel rotiert. http://de.wikipedia.org/wiki/Eulersche_Winkel
void rotateV(struct fp_vector *v, float* delta)
{
	struct fp_vector v_tmp;
	v_tmp = *v;
	
	
	v->Z -= delta[ROLL]  * v_tmp.X + delta[PITCH] * v_tmp.Y;
	v->X += delta[ROLL]  * v_tmp.Z - delta[YAW]   * v_tmp.Y;
	v->Y += delta[PITCH] * v_tmp.Z + delta[YAW]   * v_tmp.X;
	
	/*
	 //Update: Achtung: Z-Achse zeigt nach unten!
	 //Laut Wikipedia, msste das hier gelten:
	 v->Z += delta[ROLL]  * v_tmp.X - delta[PITCH] * v_tmp.Y;
	 v->X += - delta[ROLL]  * v_tmp.Z + delta[YAW]   * v_tmp.Y;
	 v->Y += delta[PITCH] * v_tmp.Z - delta[YAW]   * v_tmp.X;
	 */
}

int16_t _atan2(float y, float x)
{
	
	float z;
	z = y / x;
	
	int16_t zi;
	zi = abs((int16_t) (z * 100));
	
	int8_t y_neg = fp_is_neg(y);
	
	if ( zi < 100 ){
		if (zi > 10)
			z = z / (1.0f + 0.28f * z * z);
		if (fp_is_neg(x)) {
			if (y_neg) z -= M_PI;
			else z += M_PI;
		}
	} else {
		z = (M_PI / 2.0f) - z / (z * z + 0.28f);
		if (y_neg) z -= M_PI;
	}
	z *= (180.0f / M_PI * 10);
	return z;
}

//Test lieferte annehmbare Werte im Bereich [-2000,2000] / 0.1Grad
void getEstimatedAttitude()
{
	uint8_t axis;
	static int32_t accMag;
	static float accLPF[3];
	static uint16_t previousT;
	uint16_t currentT;
	float scale, deltaGyroAngle[3];
	//int16_t ACC_VALUE;
	
	currentT = micros();
	
	scale = (currentT - previousT)*GYRO_SCALE;
	previousT = currentT;
	
	accMag = 0;
	
	// Initialization
	for (axis = 0; axis < 3; axis++) {
		deltaGyroAngle[axis] = gyroADC[axis] * scale;
		accLPF[axis] = accLPF[axis] * (1.0f - (1.0f/ACC_LPF_FACTOR)) + accADC[axis] * (1.0f/ACC_LPF_FACTOR);
		
		//LPF-Variable, falls LPF gewnscht:
		//accSmooth[axis] = accLPF[axis];
		//#define ACC_VALUE accSmooth[axis]
		
		//mchte man keinen LPF-Filter verwenden (Hardware LPF 5Hz gesetzt):
		accSmooth[axis] = accADC[axis];
    #define ACC_VALUE accADC[axis]
		
		accMag += (int32_t)ACC_VALUE*ACC_VALUE; //~"Betrag ohne Wurzel" des Beschleunigungsvektors 
	}
	
	accMag = accMag*100/((int32_t)acc_1G*acc_1G); //auf g normierte Beschleunigungsstrke
	
	//Der Ortsvektor wird um deltaGyroAngle gedreht (am Anfang war er 0) //Update: Sollte der Vektor nicht am Anfang in eine Einheitsrichtung zeigen? z.B. z-Achse: (0,0,1)
	rotateV(&EstG.V, deltaGyroAngle);
	
	
	if ( abs(accSmooth[ROLL])<acc_25deg && abs(accSmooth[PITCH])<acc_25deg && accSmooth[YAW]>0) {
		SMALL_ANGLES_25 = 1;
	} else {
		SMALL_ANGLES_25 = 0;
	}
	
	
	//Anwenden des Zusatzfilters (Gyro drift correction)
	//Wenn accMag innerhalb der Grenzen liegt (0.6g < accMag < 1.4g, dh es sind nur kleine Beschleunigungen) und der Accvektor innerhalb der Grenzen liegt (SMALL_ANGLES), wird der Accelerometer in der Winkelbestimmung bercksichtigt
	//Ansonsten wird der Filter bersprungen und der Accelerometer hat keinen Einfluss auf die Winkelbestimmung
	//hier wird also auch die Nulllage bestimmt. Befindet sich der NanoQDC in der Horizontallage, so wird ein z-Eintrag des Ortsvektor durch die Erdbeschleunigung generiert.
	//Jede nderung aus der Ruhelage fhrt deshalb sofort zu einem Eintrag in den x und y Beschleunigungsrichtungen, da nun die Erdbeschleunigung Einfluss auf diese Gren hat
	
	
	if ( ( 36 < accMag && accMag < 196 ) || SMALL_ANGLES_25 )
		for (axis = 0; axis < 3; axis++) {
			int16_t acc = ACC_VALUE;
			EstG.A[axis] = (EstG.A[axis] * GYR_CMPF_FACTOR + acc) * INV_GYR_CMPF_FACTOR;
		}
	
	// Attitude of the estimated vector
	angle[ROLL]  =  _atan2(EstG.V.X, EstG.V.Z);
	angle[PITCH] =  _atan2(EstG.V.Y, EstG.V.Z); //Winkel in 0.1Grad Einheiten
	
	//ACC
	//Pitch nach hinten -angle[PITCH]
	//pitch nach vorne +angle[PITCH]
	//Roll nach links -angle[ROLL]
	//Roll nach rechts +angle[ROLL]
	
	//GYRO
	//Pitch nach hinten -angle[PITCH]
	//Pitch nach vorne +angle[PITCH]
	//Roll nach links -angle[ROLL]
	//Roll nach rechts +angle[ROLL]
}

void computeIMU() 
{
	uint8_t axis;
	static int16_t gyroADCprevious[3] = {0,0,0};
	int16_t gyroADCp[3];
	int16_t gyroADCinter[3];
	static uint32_t timeInterleave = 0;
	
	//Gyro und ACC Werte lesen
	ACC_getADC();
	getEstimatedAttitude(); //berechnet die "geschtzten" Winkel und LPF
	Gyro_getADC();
	
    for (axis = 0; axis < 3; axis++)
		gyroADCp[axis] =  gyroADC[axis];
    
	timeInterleave = micros();	//micros() gibt die Anzahl der vergangenen Mikrosekunden zurck, seitdem der Quadro eingeschaltet wurde
	
	annexCode(); //Code um dynamische PID Werte und rcCommand zu erzeugen, dynamische PID-Werte vorerst weglassen?
	
	if ((micros()-timeInterleave)>650) {
		annex650_overrun_count++;	//wenn der Annexcode lnger als 650mus dauert, Fehleranzahl erhhen -> debug
    } else {
		//while((micros()-timeInterleave)<650) ; //empirisch, Zeitverzgerung zwischen zwei aufeinanderfolgenden Lesevorgngen, bleibt solange in der Schleife bis 650mus vergangen sind
    }
	
	//Gyro-Werte nach 650mus lesen		// Sa hier die Gyros erneut im gleichen Durchgang auslesen???
    Gyro_getADC();
	
	//gewichtetes Mittel der Gyrowerte
    for (axis = 0; axis < 3; axis++) {
		gyroADCinter[axis] =  gyroADC[axis]+gyroADCp[axis];
		// empirical, we take a weighted value of the current and the previous values
		gyroData[axis] = (gyroADCinter[axis]+gyroADCprevious[axis])/3;
		gyroADCprevious[axis] = gyroADCinter[axis]/2;
    }
	
	//Gyro Smoothing
    #ifdef GYRO_SMOOTHING
    static int16_t gyroSmooth[3] = {0,0,0};
    for (axis = 0; axis < 3; axis++) {
		gyroData[axis] = (int16_t) ( ( (int32_t)((int32_t)gyroSmooth[axis] * (conf.Smoothing[axis]-1) )+gyroData[axis]+1 ) / conf.Smoothing[axis]);
		gyroSmooth[axis] = gyroData[axis];
	}
    #endif
}
