/* 
 * config.c
 * 
 * Configurate the Microcontroller.
 *
 * Created by Matthias Kesenheimer on 17/07/2012.
 * Copyright 2012 Matthias Kesenheimer. All rights reserved.
 *
 * Functions:
 *	void config()
 *	void calibrate()
 */

uint8_t a;

void configFirstRun()
{
	//Konfigurationshinweise:
	//P-Wert erhhen bis unkontrollierte Oszillationen beginnen, dann solange verringern bis Oszillationen verschwinden
	//I-Wert solange verndern, bis der Copter nicht mehr zum Sollwert zurckkehrt (fngt an zu driften), dann langsam erhhen
	//D-Wert erhhen bis das Ausgleichen abrupter nderungen aus der Gleichgewichtslage zu lange dauert, dann langsam erniedrigen
	
	//Merke:
	//Oszillationen -> P-Wert niedriger, oder D-Wert grer
	//Drift -> I-Wert erhhen
	//Kein Ausgleich dramatischer nderungen -> D-Wert kleiner
	
	//P: Proportional: Gegenverstrkung der Winkelgeschwindigkeit, regelt proportional zu einem Fehler
	//I: Integral: Gegenverstrkung des Winkels, wenn der Fehler konstant bleibt, wird die Verstrkung so lange erhht, bis eine Verbesserung zu verzeichnen ist
	//D: Differential: Verstrkung der Winkelbeschleunigung, wird dieser Wert erhht, regelt der Copter schneller auf seinen Soll-Wert zurck, der D-Wert "sagt den Regelwert voraus", je hher die nderung des Fehlers ist, desto grer muss die Anpassung sein.
	
	//Empfehlung von http://wiki.rc-heli-fan.org/index.php/Wii_Copter#Quadrocopter
	//p8 <= 30 fr Motorabstand < 30cm (fr grere Motorabstnde p8 = 3.0 - 3.5
	//i8 = 10 - 50, um maximal 5 erhhen
	//d8 = 5 - 25 
	
	conf.P8[ROLL] = 40; //40 //Roll Bewegung (Drehung um x-Achse)
	conf.I8[ROLL] = 30; //30
	conf.D8[ROLL] = 23; //23
	
	conf.P8[PITCH] = 40; //40 //Nick Bewegung (Drehung um y-Achse)
	conf.I8[PITCH] = 30; //30
	conf.D8[PITCH] = 23; //23
	
	
	conf.P8[YAW] = 85; //90 //Gier Bewegung (Drehung um z-Achse)
	conf.I8[YAW] = 45; //65
	conf.D8[YAW] = 0;  //0
	
	conf.P8[PIDLEVEL] = 70; //70 
	conf.I8[PIDLEVEL] = 10;	//10
	conf.D8[PIDLEVEL] = 100; //100
	
	
	conf.rcRate8 = 90; 
	conf.rcExpo8 = 65;
	conf.rollPitchRate = 0;
	conf.yawRate = 0;
	conf.dynThrPID = 0;
	
	conf.thrMid8 = 50; 
	conf.thrExpo8 = 0;
	
	conf.angleTrim[0] = 0;
	conf.angleTrim[1] = 0;
    
    #ifdef GYRO_SMOOTHING
    conf.Smoothing[3] = GYRO_SMOOTHING;
    #endif
    
	conf.checkNewConf = EEPROM_CONF_VERSION;
}

void config()
{
	
	//Wenn das Programm auf den Mikrokontroller zum ersten mal geladen wird,
	//mssen die Startwerte ins EEprom von Hand mit folgendem Code geschrieben werden.
	//Dieser Code muss mit dem Setzen von FIRST_RUN auf 0 bei jedem nchsten
	//Mal auskommentiert werden. Gleichzeitig kann mit FIRST_RUN = 1 das Eprom
	//auf die Startwerte zurckgesetzt werden.
	
	#if FIRST_RUN == 1
		//Startwerte ins EProm schreiben
		//configFirstRun();
		//EEpromWriteBlock(&conf, (void*)&startAd, sizeof(conf));
	#endif

	/*  Wenn EEPROM_CONF_VERSION bereits im EEPROM steht, wird sofort gelesen.
		Wenn EEPROM_CONF_VERSION nicht vorhanden ist (andere Version oder FF), wird das EEPROM initialisiert.
	*/
	//if (EEpromReadByte(&conf.checkNewConf) != EEPROM_CONF_VERSION)	{
		//configFirstRun();
		//EEpromWriteBlock(&conf, (void*)&startAd, sizeof(conf));		
		//
		///* Anzeigen, wenn EEPROM neu geschrieben wird */
		//PORTC.OUTSET |= (1<<PIN4);		//LED rot
		////delay(500);
		//for (a=0; a<4; a++) {_delay_loop_2(65535);}	// ca. 500 ms
		//PORTC.OUTCLR |= (1<<PIN4);		
	//}
	
	
	EEpromReadBlock(&conf, (void*)&startAd, sizeof(conf));
		
	/* Anzeigen, wenn EEPROM gelesen wird */
	PORTC.OUTSET |= (1<<PIN5);		//LED blau
	//delay(200);
	for (a=0; a<2; a++) {_delay_loop_2(65535);}	// ca. 200 ms
	PORTC.OUTCLR |= (1<<PIN5);
	
	/* Ablauf mit Eprom	 
	 sobald von der GUI andere Configwerte empfangen werden conf.checkNewConf++ und die neuen Werte gleich ins Eprom schreiben.	 
	 das nchste mal, wenn beim Starten config() aufgerufen wird, wird die erste Abfrage bersprungen und 	 
	 if conf.checkNewConf >= conf.versionNumber	 
	 */
	
	acc_25deg = acc_1G * 0.423;
	
	uint8_t i;
	for (i=0; i<8;i++)
		rcData[i] = 0; //Startwerte setzen
	
	for(i=0;i<6;i++) {
		lookupPitchRollRC[i] = (2500+conf.rcExpo8*(i*i-25))*i*(int32_t)conf.rcRate8/2500;
	}
	for(i=0;i<11;i++) {
		int16_t tmp = 10*i-conf.thrMid8;
		uint8_t y = 1;
		
		if (tmp>0)
			y = 100-conf.thrMid8;
		if (tmp<0)
			y = conf.thrMid8;
		
		lookupThrottleRC[i] = 10*conf.thrMid8 + tmp*( 100-conf.thrExpo8+(int32_t)conf.thrExpo8*(tmp*tmp)/(y*y) )/10; // [0;1000]
		lookupThrottleRC[i] = MINTHROTTLE + (int32_t)(MAXTHROTTLE-MINTHROTTLE)* lookupThrottleRC[i]/1000;            // [0;1000] -> [MINTHROTTLE;MAXTHROTTLE]
	}
	
	EstG.V.X = 0;
	EstG.V.Y = 0;
	EstG.V.Z = 530;
}


void calibrate()
{
	calibratingG = 400; //Anzahl der Kalibrationsdurchgnge Gyro
	calibratingA = 400;
	maxStepsGA = 400;
}
