/* 
 * def.c
 * 
 * Defining all Variables.
 *
 * Created by Matthias Kesenheimer on 17/07/2012
 * Copyright 2012 Matthias Kesenheimer. All rights reserved
 *
 */
#include "serial_2.h"

#ifndef F_CPU
/*! \brief Define default CPU frequency, if this is not already defined. */
#define F_CPU 32000000UL
#endif

#ifndef	delay
#define delay(X) _delay_ms(X)
#endif

//#define NEW_ALGOS	1	// Teile aus Matthias' neuen Sourcen
#define Debugging 0		// LED-Ansteuerung anstelel der Motoren

//Alias
#define ROLL       0	//X
#define PITCH      1	//Y
#define YAW        2	//Z
#define THROTTLE   3	//Gas
#define AUX1       4
#define AUX2       5
#define AUX3       6
#define AUX4       7
#define PIDLEVEL   7
#define PIDITEMS   10

//Senden und Empfangen
#define PID		   0
#define NR		   1
#define NOT		   2
#define RECON	   1
#define RECOFF	   0

//entscheidet was an die GUI gesendet werden soll
//static uint8_t plot = NOT;
/* Zeug fr Matthias Comm-Routine
static uint8_t receiveOn = RECOFF;
static uint8_t iter = 0;
static uint8_t next = 1;
static uint8_t before = 0;
Zeug fr Matthias Comm-Routine 
static uint8_t whichPIDToSend = 0;
static uint8_t motorsActive = 0;*/

//512 Die Erdbeschleunigung muss wegskaliert werden,
//die Z-Beschleunigungswerte haben immer einen
//Offset von ungefaehr 512 = 1g
static uint16_t acc_1G = 530;
static int16_t  acc_25deg;

//Konfigurationsvariablen, ggf ins EEPROM zur Speicherung schreiben
static uint8_t dynP8[3], dynD8[3]; //dynamische P und D Werte zur Regelung, werden in annex_Code() berechnet.
static struct
{
	uint8_t checkNewConf;
	uint8_t P8[PIDITEMS], I8[PIDITEMS], D8[PIDITEMS];
	uint16_t activate[CHECKBOXITEMS];
	uint8_t powerTrigger1;
	uint8_t dynThrPID;
	uint8_t rcRate8;
	uint8_t rcExpo8;
	uint8_t rollPitchRate;
	uint8_t yawRate;
	uint8_t thrMid8;
	uint8_t thrExpo8;
	int16_t angleTrim[2];
	int16_t accZero[3];
	uint8_t Smoothing[3];
} conf;

static uint8_t EEMEM startAd[sizeof(conf)];
#define FIRST_RUN 0 //auf Null setzen, wenn das Programm zum ersten Mal auf den QDC geladen wurde. Ist nur dazu da, das EEPROM zu initialiseren/formatieren. vielleicht kann man das auch irgendwie anders lsen!
#define EEPROM_CONF_VERSION 164

//I2C Einstellungen
#define CPU_SPEED	32000000
#define BAUDRATE	400000
#define SLAVE_ADDRESS    0x69 //Adresse des MPU6050
#define TWI_BAUDSETTING TWI_BAUD(CPU_SPEED, BAUDRATE)	// = 35

//USART Einstellungen
volatile uint8_t packetBuffer[31];
int16_t receivedData = 0;

// WATCHDOG
// Select only one
//#define WATCHDOG_TIMEOUT        WDT_PER_8CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_16CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_32CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_64CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_128CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_256CLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_512CLK_gc
#define WATCHDOG_TIMEOUT        WDT_PER_1KCLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_2KCLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_4KCLK_gc
//#define WATCHDOG_TIMEOUT        WDT_PER_8KCLK_gc

//TODO: #ifdef verwenden
//Empfnger Baudrates
#define DSM2 115200
#define DSL 38400

//hier Receiver whlen:
// -->> #define RECEIVER DSL

#define USART_BAUDRATE RECEIVER //38400 fr Domis Quad, 115200 fr meinen Quad
#define BAUD_PRESCALE (((F_CPU / (USART_BAUDRATE * 16UL))) - 1)

//DEBUG UART //Daten an die GUI
#define USART_BSCALEC0 0
#define USART_BAUDRATEC0  115200 //19200// //38400 //115200 //57600
#define BAUD_PRESCALEC0 ((F_CPU / ((int16_t)(pow (2.0, USART_BSCALEC0)) * (USART_BAUDRATEC0 * 16UL)))  - 1)

//I2C
TWI_Master_t twiMaster;
static uint8_t sendbuffer[2];
//static register8_t ra;	//temporaere Registeradresse zum Senden der I2C Daten
static uint8_t ra;	//temporaere Registeradresse zum Senden der I2C Daten, das muss nicht als Register definiert sein! Sa

//Inflight Calibration
#define INFLIGHT_ACC_CALIBRATION 1
static uint16_t InflightcalibratingA = 0;
static int16_t AccInflightCalibrationArmed;
static uint16_t AccInflightCalibrationMeasurementDone = 0;
static uint16_t AccInflightCalibrationSavetoEEProm = 0;
static uint16_t AccInflightCalibrationActive = 0;

/* Failsafe */
//#define FAILSAFE           1                       // uncomment  to activate the failsafe function
#define FAILSAVE_DELAY     10                     // Guard time for failsafe activation after signal lost. 1 step = 0.1sec - 1sec in example
#define FAILSAVE_OFF_DELAY 200                    // Time for Landing before motors stop in 0.1sec. 1 step = 0.1sec - 20sec in example
#define FAILSAVE_THROTTLE  (MINTHROTTLE + 200)    // Throttle level used for landing - may be relative to MINTHROTTLE - as in this case
#define STABLEPIN_ON               ;
#define STABLEPIN_OFF              ;

/* LEDs */
//#define LEDPIN_PINMODE             pinMode (13, OUTPUT);pinMode (30, OUTPUT);
#define LEDPIN_TOGGLE             // PORTC.OUTTGL |= (1<<PIN5);
#define LEDPIN_ON                 // PORTC.OUTSET |= (1<<PIN5);
#define LEDPIN_OFF                // PORTC.OUTCLR |= (1<<PIN5);
//#define BUZZERPIN_PINMODE          pinMode (32, OUTPUT);
//#if defined(BUZZER)
static uint8_t  buzzerFreq;         // delay between buzzer ring
//#endif
#define BUZZERPIN_ON               //PORTC.OUTSET |= (1<<PIN4);
#define BUZZERPIN_OFF              //PORTC.OUTCLR |= (1<<PIN4);


#define MINCOMMAND  888

//PWM
#define MOTOR_CNT 4
//#define PIDMIX(X,Y,Z) (rcCommand[THROTTLE] + axisPID[ROLL]*X + axisPID[PITCH]*Y + axisPID[YAW]*Z)
static uint16_t motor[MOTOR_CNT];		//Sollwerte der Motoren aus den Algorithmen fr GUI, [1000..2000]
static uint16_t motor_s[MOTOR_CNT];		//Sollwerte fr Timer fr PWM, [16200..32400]

//gyro+acc IMU
typedef struct fp_vector {
	float X;
	float Y;
	float Z;
} t_fp_vector_def;

typedef union {
	float   A[3];
	t_fp_vector_def V;
} t_fp_vector;

static int16_t gyroADC[3] = {0,0,0};
static int16_t accADC[3] = {0,0,0};
static int16_t accSmooth[3] = {0,0,0};
static int16_t temperatur;
static int16_t gyroData[3] = {0,0,0};
static int16_t gyroZero[3] = {0,0,0};
static int16_t angle[2]    = {0,0};	// Absoluter Winkel in 0.1 Grad Einheiten, 180 deg = 1800
static uint16_t calibratingG = 400; 	//Anzahl der Kalibrationsdurchgaenge Gyro
static uint16_t calibratingA = 400;
static uint16_t maxStepsGA = 400;

#if defined(BUZZER)
	static uint8_t  toggleBeep = 0;
#endif

static int32_t calibrationSum[3] = {0,0,0};
static uint8_t SMALL_ANGLES_25 = 1;
static t_fp_vector EstG; //aktueller Ortsvektor
#define fp_is_neg(val) (( ( (uint8_t*) &val)[3] & 0x80) != 0)

//Zeitmanagement
volatile uint16_t ms10;			// Zhler fr 10ms-Happen, die im OVF-IRQ von Timer E0 incrementiert werden
static int16_t  annex650_overrun_count = 0;
static uint32_t currentTime = 0;
static uint16_t previousTime = 0;
static uint16_t cycleTime = 0;  // this is the number in micro second to achieve a full loop,
//it can differ a little and is taken into account in the PID loop

//Empfaengersettings
static volatile uint8_t channel;	//aktueller Kanal beim Einlesen der Receiver-Werte
#define MIDRC 1500 //Falls der Empfnger einen anderen Offset(Null)-Wert hat, hier ndern
volatile uint16_t rcValue[8] = {1502, 1502, 1000, 1502, 1502, 1502, 1502, 1502}; // interval [1000;2000]
//static uint8_t rcChannel[8] = {ROLL, PITCH, YAW, THROTTLE, AUX1, AUX2, AUX3, AUX4}; //DSL-4Top Kanalzuordnung
static uint8_t rcChannel[8] = {PITCH,YAW,THROTTLE,ROLL,AUX1,AUX2,AUX3,AUX4}; //DSM2 Kanalzuordnung DX6i
static int16_t rawRCData[9];	// unnormierte, im IRQ aus dem Timer CNT ausgelesene Werte, analog zu den Kanalwerten
static int16_t rcData[8];          // interval [1000;2000], aktuelle rcDaten, gemittelt
static int16_t rcCommand[4];       // interval [1000;2000] for THROTTLE and [-500;+500] for ROLL/PITCH/YAW
static int16_t lookupPitchRollRC[6];	// lookup table for expo & RC rate PITCH+ROLL
static int16_t lookupThrottleRC[11];	// lookup table for expo & mid THROTTLE
#define MINTHROTTLE 1000		//1150 Minimalgas der Motoren, Standgas
#define MAXTHROTTLE 2000	//1850 //Maximalwert der Motoren
#define MINCHECK 1000	//1100
#define MAXCHECK 1900	//1900
#define BREAKPOINT 1500 //1500

//PID-Settings
// default POSHOLD control gains
static int16_t axisPID[3];

/********************************************************************/
/****                             Buzzer                         ****/
/********************************************************************/
//#define BUZZER
//#define RCOPTIONSBEEP        //uncomment this if you want the buzzer to beep at any rcOptions change on channel Aux1 to Aux4
//#define ARMEDTIMEWARNING 330  // Trigger an alarm after a certain time of being armed [s] to save you lipo (if your TX does not have a countdown)

/********************************************************************/
/****           battery voltage monitoring                       ****/
/********************************************************************/
/* for V BAT monitoring
    after the resistor divisor we should get [0V;5V]->[0;1023] on analog V_BATPIN
    with R1=33k and R2=51k
    vbat = [0;1023]*16/VBATSCALE
    must be associated with #define BUZZER ! */
#define VBAT              // uncomment this line to activate the vbat code
#define VBATSCALE       1510 // (*) change this value if read Battery voltage is different than real voltage
#define VBATNOMINAL     42 // 4,20 V   full battery nominal voltage - only used for lcd.telemetry
#define VBATLEVEL1_3S   36 // 3,57 V      // scaled to 1S cell
#define VBATLEVEL2_3S   34 // 3,43 V
#define VBATLEVEL3_3S   32 // 3,30 V
#define VBATLEVEL_CRIT  30 // 3,10 V - critical condition: if vbat ever goes below this value, permanent alarm is triggered
    
#define VBATLEVEL_WARN1 107 // (*) 10,7V
#define VBATLEVEL_WARN2  99 // (*) 9.9V
#define NO_VBAT          16  // (*) Avoid beeping without any battery
#define V_BATPIN		0	// beim xNQ liegt Akku an Port A -> Pin 0

static uint8_t Lowvoltage = 0;	// Flag fr Unterspannung


/****           internal frequencies                             ****/
/* frequenies for rare cyclic actions in the main loop, depend on cycle time
    time base is main loop cycle time - a value of 6 means to trigger the action every 6th run through the main loop
    example: with cycle time of approx 3ms, do action every 6*3ms=18ms
    value must be [1; 65535] */
//    #define LCD_TELEMETRY_FREQ 23       // to send telemetry data over serial 23 <=> 60ms <=> 16Hz (only sending interlaced, so 8Hz update rate)
//    #define LCD_TELEMETRY_AUTO_FREQ 667 // to step to next telemetry page 667 <=> 2s
#define PSENSORFREQ 6               // to read hardware powermeter sensor 6 <=> 18ms
#define VBATFREQ PSENSORFREQ        // to read battery voltage - keep equal to PSENSORFREQ unless you know what you are doing


//-----------------------------
//Low Pass filter Einstellungen
//-----------------------------

/* Set the Low Pass Filter factor for ACC */
/* Erhhen dieses Werts verringert das Rauschen des ACCs, aber verzgert den ACC Einfluss */
/* Niedrigerer Wert, hheres Rauschen, schnellerer Einfluss */
#define ACC_LPF_FACTOR 100 //100

/* Set the Gyro Weight for Gyro/Acc complementary filter */ //Ergaenzender Filter
/* Erhhen dieses Werts reduziert und verzgert den ACC Einfluss */
/* Niedrigerer Wert, mehr ACC Einfluss */ 
#define GYR_CMPF_FACTOR 100.0f //400
#define INV_GYR_CMPF_FACTOR   (1.0f / (GYR_CMPF_FACTOR  + 1.0f))

//#define GYRO_SCALE ((2380 * M_PI)/((32767.0f / 4.0f ) * 180.0f * 1000000.0f)) //should be 2279.44 but 2380 gives better result
// +-2000/sec deg scale
#define GYRO_SCALE ((200.0f * M_PI)/((32768.0f / 5.0f / 4.0f ) * 180.0f * 1000000.0f) * 2.0f)
//Original: #define GYRO_SCALE ((200.0f * M_PI)/((32768.0f / 5.0f / 4.0f ) * 180.0f * 1000000.0f) * 1.5f)     
// +- 200/sec deg scale
// 1.5 is empirical, not sure what it means
// should be in rad/sec 
// Small angle approximation

//Gyrowerte mitteln - nur im uersten Notfall benutzen
//#define GYRO_SMOOTHING {20, 20, 3}


#define ssin(val) (val)
#define scos(val) (1.0f - (val)*(val)/2.0f)

#ifndef max
#define max(a, b) ( ((a) > (b)) ? (a) : (b) )
#endif

#ifndef min
#define min(a, b) ( ((a) < (b)) ? (a) : (b) )
#endif

#ifndef constrain
#define constrain(Value, Min, Max) ( ((Value) < (Min)) ? (Min) : ((Value) > (Max)) ? (Max) : (Value) )
#endif

/* GPS */
static int16_t  GPS_angle[2] = { 0, 0};                      // it's the angles that must be applied for GPS correction
