/* 
 * init.c
 * 
 * Initializing the Microcontroller.
 *
 * Created by Matthias Kesenheimer on 17/07/2012.
 * Copyright 2012 Matthias Kesenheimer. All rights reserved.
 *
 * Functions:
 * 	void clock_init(void)
 *	void timerE0_init(void)
 *	void PWM_init(void)
 *	void ports_init(void)
 *	void Gyro_init()
 *	void ACC_init()
 *	void initSensors()
 *	void usart_init()
 *
 */

uint8_t a;

void clock_init()
{
	OSC.CTRL = OSC_RC32MEN_bm;		//aktiviert internen 32MHz Oszillator
	while(!(OSC.STATUS & OSC_RC32MRDY_bm));	//warte bis der Oszillator stabil luft
	CCP = CCP_IOREG_gc;			//deaktiviert den Interrupt fr 4 Taktzyklen -> Schutz
	CLK.CTRL = 0x01;			//Waehle den internen 32MHz Oszillator als Taktquelle aus
}

void timerE0_init() //Timer E0 loest alle 10 ms einen Interrupt aus (OVF_IRQ)
{
	//Timer E0 initialisieren
	TCE0.CTRLA = TC_CLKSEL_DIV64_gc;	//Prescaler auf 64 -> 0,5 MHz Clk fr den Timer -> 2s Resolution
	TCE0.CTRLB = 0x00;					//Timer im "Normalmode"
	//TCE0.INTCTRLA = 0x01;				//Interrupt hat nun niedrige Prioritt
	TCE0.INTCTRLA = 0x02;				//Interrupt hat nun mittlere Prioritt
	//TCE0.INTCTRLA = 0x03;				//Interrupt hat nun hohe Prioritt
	TCE0.PER = 4999;					// free running mit 2s, wir lesen nur den Zhlerstand aus
	// zum Mesen mit dem Oszi:
	//TCE0.CTRLB = ((1<< TC0_CCDEN_bp) | TC0_WGMODE0_bm | TC0_WGMODE1_bm);	// Timer klappert mit Pin33 (OC0D von Port E)
	//TCE0.CCD = 0x08;														// Timer klappert mit Pin33 (OC0D von Port E)
	
	ms10 = 0;				//variable, die die vergangenen Millisekunden in Zehnerschritten zaehlt
}

void timerC0_init() // Receiver-IRQ, Abtastung des SUM-Impulszugs (alle Kanle seriell an einem Pin, 
{					//Timer C0 loest nach 4 ms einen Interrupt aus, wenn kein Kanalimpuls vom Receiver kam -> Chan = 0
					// Ansonsten zhlt er die vergangenen Timer-Ticks zwischen zwei Kanalimpulsen

	//Timer C0 initialisieren
	TCC0.CTRLA = TC_CLKSEL_DIV64_gc;	//Prescaler auf 64 -> 0,5 MHz Clk fr den Timer -> 2s Resolution
	TCC0.CTRLB = 0x00;					//Timer im "Normalmode"
	//TCC0.INTCTRLA = 0x01;				//Interrupt hat nun niedrige Prioritt
	//TCC0.INTCTRLA = 0x02;				//Interrupt hat nun mittlere Prioritt
	TCC0.INTCTRLA = 0x03;				//Interrupt hat nun hohe Prioritt
	TCC0.PER = 2050;					// alle 4ms OVL-IRQ -> kommt er hier hin, ist eine Impulsfolge vom Receiver um
										// und es geht wieder mit Kanal 1 los
	channel = 0;
}

void PWM_init()
{
	//Timer D0 als PWM initialisieren
	TCD0.CTRLA = TC_CLKSEL_DIV2_gc;		//Prescaler auf 2 (0x02) -> 16 MHz
	TCD0.CTRLB = 0xf3;		//Single Slope und PWM an allen PWM Ausgaengen freischalten
	TCD0.PER = 40000;	//0xefff;		//ganzer Zaehlerbereich wird verwendet
	
	TCD1.CTRLA = TC_CLKSEL_DIV2_gc;		//Prescaler auf 2 (0x02) -> 16 MHz
	TCD1.CTRLB = 0xf3;		//Single Slope und PWM an allen PWM Ausgaengen freischalten
	TCD1.PER = 40000;	//0xefff;		//ganzer Zaehlerbereich wird verwendet

	//Motorzuordnungen Quad   |  Layout der Leiterplatte  |  Willas BASCOM Code  |	 MWC-Belegung	|
	//vorne links  : 3        |        (1)    (2)	  	  |      (3)    (2)      |	  [0]    [3]	|
	//vorne rechts : 2        |           \  /		  	  |         \  /         |		 \  /		|
	//hinten rechts: 1        |           /  \		  	  |         /  \         |		 /  \		|
	//hinten links : 4        |        (4)    (3)	  	  |      (4)    (1)      |	  [1]	 [2]	|

	motor[0] = MINCOMMAND;
	motor[1] = MINCOMMAND;
	motor[2] = MINCOMMAND;
	motor[3] = MINCOMMAND;
}

void ports_init()
{
	//initialisierung der Motorenausgnge			// Sa
	PORTD.DIRSET = PIN5_bm;		//Motor 1 (xNQ Eagle), links vorne
	PORTD.DIRSET = PIN4_bm;		//Motor 2 (xNQ Eagle), rechts vorne
	PORTD.DIRSET = PIN3_bm;		//Motor 3 (xNQ Eagle), rechts hinten
	PORTD.DIRSET = PIN2_bm;		//Motor 4 (xNQ Eagle), links hinten
	//PORTD.DIR |= (1<<PIN5);		//Motor 1 (xNQ Eagle), links vorne
	//PORTD.DIR |= (1<<PIN4);		//Motor 2 (xNQ Eagle), rechts vorne
	//PORTD.DIR |= (1<<PIN3);		//Motor 3 (xNQ Eagle), rechts hinten
	//PORTD.DIR |= (1<<PIN2);		//Motor 4 (xNQ Eagle), links hinten
	//
	//LEDs definieren			// Sa
	PORTA.DIR |= (1<<PIN6);		// LED vorne (blau) Pos-LED
	PORTA.DIR |= (1<<PIN7);		// LED hinten (rot) Pos-LED
	PORTC.DIR |= (1<<PIN4);		// LED ROT (Status)
	PORTC.DIR |= (1<<PIN5);		// LED BLAU (Status)
	PORTD.DIR |= (1<<PIN6);		// LED GELB (Status)
	
	//TWI Ausgnge - Noetig?
	PORTE.DIRSET = PIN0_bm;      // Als Ausgang  TWI SDA
	PORTE.DIRSET = PIN1_bm;      // Als Ausgang  TWI SCL
	
	//Testausgang fr Timer E0
	PORTE.DIRSET = PIN3_bm;
	
	//Empfnger Eingang			// Sa
	PORTB.DIRCLR = PIN2_bm;		//Receiver Eingang
	
	//GUI USARTC0 Ausgang			// Sa
	PORTC.OUTSET = PIN3_bm;			// Sa
	PORTC.DIRSET = PIN3_bm;			//PortC Pin3 TxD
	PORTC.DIRCLR = PIN2_bm;			//PortC Pin2 RxD

	PORTA.DIRCLR = PIN0_bm;		// Eingang fr VBAT
}

void ADC_init()
{	
	ADCA.CTRLA |= ADC_ENABLE_bm; 	// enable adc 
	ADCA_CTRLB |= (ADC_FREERUN_bm | ADC_RESOLUTION_12BIT_gc);	// free running conversion mode, unsigned, 12bit resolution
	ADCA_REFCTRL |= ADC_REFSEL_VCC_gc;							// VCC/1.6 ~ 2,0625 V
	//' Ref = 2,0625 V <-> 4096
	//' 0,50354 mV / Digit -> 3,8 V <->
	//' gemessene Werte: 4,2 V -> 2047, 3,8 V  -> 1996, 3,6 V -> 1904
	ADCA_PRESCALER |= ADC_PRESCALER_DIV256_gc;					// DIV256 -> 125 kHz sample rate
	
	ADCA_CH0_CTRL |= ADC_CH_INPUTMODE_SINGLEENDED_gc;
	ADCA_CH0_MUXCTRL = 0;
	//ADCA_EVCTRL |= (ADC_SWEEP0_bm |	ADC_EVSEL0_bm | ADC_EVACT0_bm);
	//
	ADCA_Cal();			// Kalibrierungswerte aus signature row lesen und in ADCA_CAL ablegen (Danke Kampi!)
	
	//ADCA_CH0_CTRL |= ADC_CH_START_bm;							// starten, free running
}

void Gyro_init()
{
	//PWR_MGMT_1
	sendbuffer[0] = 0x6b;	//Registeradresse PWR_MGMT_1
	sendbuffer[1] = 0x80;	//Device Reset
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2); //ohne & meckert der Compiler nicht mehr, ausprobieren!
	while (twiMaster.status != TWIM_STATUS_READY){}
	
	//_delay_ms(10);
	_delay_loop_2(65535);	// 8,2 ms
	
	//PWR_MGMT_1
	sendbuffer[0] = 0x6b;	//Registeradresse PWR_MGMT_1
	sendbuffer[1] = 0x03;	//DEVICE_RESET = 0, SLEEP = 0, CYCLE = 0, TEMP_DIS = 0, CLKSEL = PLL with z axis gyro reference
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
	
	//DLPF
    //#define MPU6050_DLPF_CFG   0 //256Hz
	//#define MPU6050_DLPF_CFG   1 //188Hz
	//#define MPU6050_DLPF_CFG   2 //98Hz
	#define MPU6050_DLPF_CFG   3 //42Hz
	//#define MPU6050_DLPF_CFG   4 //20Hz
	//-->>#define MPU6050_DLPF_CFG   5 //10Hz
	//#define MPU6050_DLPF_CFG   6 //5Hz
	
	//CONFIG
	sendbuffer[0] = 0x1a;
	sendbuffer[1] = MPU6050_DLPF_CFG;	//EXT_SYNC_SET = 0, Digital Low Pass Filter = 0 -> 260Hz Bandbreite Acc, 256Hz Bandbreite Gyro
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
	
	sendbuffer[0] = 0x1a;
	sendbuffer[1] = MPU6050_DLPF_CFG;	//EXT_SYNC_SET = 0, Digital Low Pass Filter = 0 -> 260Hz Bandbreite Acc, 256Hz Bandbreite Gyro
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
	
	sendbuffer[0] = 0x1a;
	sendbuffer[1] = MPU6050_DLPF_CFG;	//EXT_SYNC_SET = 0, Digital Low Pass Filter = 0 -> 260Hz Bandbreite Acc, 256Hz Bandbreite Gyro
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
	

	//GYRO_CONFIG
	sendbuffer[0] = 0x1b;
	sendbuffer[1] = 0x18;	//Gyro.Selftest = disabled, GYRO_CONFIG - FS_SEL = 3: Full scale set to +-2000 deg/sec
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}
}

void ACC_init()
{
	//ACCEL_CONFIG
	sendbuffer[0] = 0x1c;
	sendbuffer[1] = 0x10;	//Accel.Selftest = disabled, AccelometerFullScaleRange = +-8g (Zweithchster Aufnahmebereich), Digital High Pass Filter = disabled
	TWI_MasterWrite(&twiMaster, SLAVE_ADDRESS, &sendbuffer[0], 2);
	while (twiMaster.status != TWIM_STATUS_READY){}	
}

void initSensors()
{
	//_delay_ms(300);
	for (a=0; a<3; a++) {_delay_loop_2(65535);}	// ca. 300 ms
	Gyro_init();
	ACC_init();
	//_delay_ms(300);
	for (a=0; a<3; a++) {_delay_loop_2(65535);}	// ca. 300 ms
}

void receiver_init()
{	// das Summensignal wird in der IRQ-Routine zerlegt. Der Abstand zw. zwei fallenden Flanken ist die Kanalinformation.
	// Eine Pause im Signal grer als 3 ms ist die Trennung zw. zwei aufeinander folgenden Impulspaketen und wird durch
	// den Overflow-IRQ des Timers erkannt.

	PORTB_INTCTRL = 0x03;					// interrupt level HI
	PORTB_INT0MASK = PIN2_bm;				// Pin2 ist Eingang fr die Receiver-Impulse
	PORTB_PIN2CTRL = PORT_ISC_FALLING_gc;	// fallende Flanke lst IRQ aus
}

void usart_init()
{
	//USART zum Senden von Daten an die GUI
	
	USARTC0.BAUDCTRLA = BAUD_PRESCALEC0;
	USARTC0.BAUDCTRLB = (BAUD_PRESCALEC0 >> 8);
	
	USARTC0.CTRLA |= (USART_RXCINTLVL_LO_gc | USART_DREINTLVL_LO_gc); //Sa  USART_RXCINTLVL_LO_gc;
	USARTC0.CTRLB |= (USART_TXEN_bm | USART_RXEN_bm); //Senden und Empfangen einschalten  
	USARTC0.CTRLC |= (USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_DISABLED_gc | USART_CHSIZE_8BIT_gc); //Sa  Asynchrone bertragung, Kein Parity Bit, 8 Bit Zeichengre
	
	serialHeadRX[0] = 0;	// Bufferhandling definiert aufsetzen
	serialTailRX[0] = 0;
	
#if 0
	//Domis Quadrocopter mit DSL Empfnger
    #if RECEIVER == DSL
		USARTD1.BAUDCTRLB = (BAUD_PRESCALE >> 8);
		USARTD1.BAUDCTRLA = BAUD_PRESCALE;
		
		USARTD1.CTRLA = USART_RXCINTLVL_LO_gc; //schwache Interruptprioritt
		USARTD1.CTRLB = USART_RXEN_bm; //Empfangen einschalten
        PMIC.CTRL |= PMIC_LOLVLEX_bm; //weglassen?
		USARTD1.CTRLC = USART_CHSIZE_8BIT_gc; // 8 Bit Zeichengre
    #endif
	
	//Mein Quadrocopter mit DSM2 Empfnger
    #if RECEIVER == DSM2
		USARTD1.BAUDCTRLB = 0b11010000; //BSEL = 131 und BSCALE = -3
		USARTD1.BAUDCTRLA = 0b10000011;	// -> 115200B
		
		USARTD1.CTRLA = USART_RXCINTLVL_HI_gc; //vor dem 20.07.13: USART_RXCINTLVL_LO_gc
		USARTD1.CTRLB = USART_RXEN_bm;
		USARTD1.CTRLC = USART_CMODE_ASYNCHRONOUS_gc | USART_PMODE_DISABLED_gc | USART_CHSIZE_8BIT_gc; //Asynchrone bertragung, Kein Parity Bit, 8 Bit Zeichengre
    #endif
#endif	
//------------------------------------
}

void setup()
{
	//Initialisierungen
	clock_init();
	ports_init();
	EEpromInit();
	
	readEEPROM();
	//checkFirstTime();

	//Fuer DEBUG - Motoren ausschalten
#if 1
    PWM_init();
#endif
	
	timerE0_init(); //TimerE0 ist dafr zustndig, dass jede us ein Interrupt ausgeloest wird	
	timerC0_init();	//TimerC0 ist fr die Receiver SUM-Channel Auswertung zustndig
	TWI_MasterInit(&twiMaster, &TWIE, TWI_MASTER_INTLVL_HI_gc, TWI_BAUDSETTING);	// TWI am PortE initialisiert	
	usart_init();
	
	//Interruptlevel freigeben
	PMIC.CTRL = (PMIC_LOLVLEN_bm | PMIC_MEDLVLEN_bm | PMIC_HILVLEN_bm);
	sei();	// Interrupts global enablen
	
	PORTC.OUTSET |= (1<<PIN5);		//LED blau
	//delay(500);
	for (a=0; a<4; a++) {_delay_loop_2(65535);}	// ca. 500 ms
	PORTC.OUTCLR |= (1<<PIN5);
	
	//Wichtig: muss nach der Initialisierung der Interrupts durchgefuehrt werden!,
	//da die Initialisierung auf I2C-Interrupts zugreift
	initSensors();
	config(); //Configuration
	
	//_delay_ms(3000); //Der Quadro wartet 3 Sekunden, danach kalibriert er sich
	calibrate();
	
	receiver_init();	// Sa, PortB Pin2 als Eingang fr Receiver, IRQ bei rising edge
	
	ADC_init();			// Sa, ADC konfigurieren fr VBAT
    
    //Watchdog initialisieren
    WDT_EnableAndSetTimeout();
    //while (1){PORTA.OUTTGL |= (1<<PIN7);}; //Test	// LED hinten (rot)
	
}

